# frozen_string_literal: true

# Copyright (c) 2018 by Jiang Jinyang <jjyruby@gmail.com>
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.


require 'ciri/utils'

module Ciri
  # Core extension module for convenient
  module CoreExt

    refine(String) do
      def to_hex
        Utils.to_hex(self)
      end

      alias encode_hex to_hex

      def decode_hex
        Utils.to_bytes(self)
      end

      def keccak
        Utils.keccak(self)
      end

      def decode_big_endian
        Utils.big_endian_decode(self)
      end

      alias decode_number decode_big_endian

      def pad_zero(size)
        self.rjust(size, "\x00".b)
      end
    end

    refine(Integer) do
      def ceil_div(size)
        Utils.ceil_div(self, size)
      end

      def encode_big_endian
        Utils.big_endian_encode(self)
      end
    end

  end
end